@@ -3,7 +3,8 @@ import re
 import numpy as np
 from scipy.optimize import bisect
 from scipy.stats import norm
-
+from covid19_outbreak_simulator.utils import as_float, as_int
+from fnmatch import fnmatch
 
 class Params:
 
@@ -51,17 +52,13 @@ class Params:
             if int(ID) >= self.groups[''] or int(ID) < 0:
                 raise ValueError(f'Invalid ID {ID}: Index out of range.')
         else:
-            found = False
-            for name, size in self.groups.items():
-                if ID.startswith(name) and ID[len(name):].isnumeric():
-                    idx = int(ID[len(name):])
-                    if idx >= self.groups[name] or idx < 0:
-                        raise ValueError(
-                            f'Invalid ID {ID}: Index out of range.')
-                    found = True
-                    break
-            if not found:
+            if '_' not in ID:
+                raise ValueError(f'index or group_index is expected for individual ID: {ID} provided')
+            grp, idx = ID.rsplit('_')
+            if grp not in self.groups:
                 raise ValueError(f'Invalid ID {ID}')
+            if int(idx) >= self.groups[grp] or int(idx) < 0:
+                raise ValueError(f'Invalid ID {idx}: Index out of range.')
 
     def set_popsize(self, val):
         if not val:
@@ -69,17 +66,10 @@ class Params:
         for ps in val:
             if '=' in ps:
                 name, size = ps.split('=', 1)
-                if name[-1].isnumeric():
-                    raise ValueError(
-                        f'Group names that end with numbers are not allowed. {name}'
-                    )
             else:
                 name = ''
                 size = ps
-            try:
-                size = int(size)
-            except Exception:
-                raise ValueError(f'Incorrect popsize {ps}')
+            size = as_int(size)
             if name in self.groups:
                 raise ValueError(f'Group "{name}" has been specified before')
             self.groups[name] = size
@@ -90,45 +80,38 @@ class Params:
         for ID in val:
             self.check_id(ID)
 
+    def _set_multiplier(self, multiplier, param_name):
+        name, value = multiplier.split('=', 1)
+        value = as_float(
+            value, "Multiplier should have format name=float_value")
+        names = [x for x in self.groups.keys() if fnmatch(x, name)]
+        if not names:
+            raise ValueError(f'Invalid group name {name} in multiplier {multiplier}')
+        for name in names:
+            self.set(param_name, f'multiplier_{name}', value)
+
+
     def set_symptomatic_r0(self, val):
         if not val:
             return
         # if asymptomatic_r0 is specified, it should a number of a range...
         pars = [x for x in val if '=' not in x]
         if len(pars) == 1:
-            try:
-                self.set('symptomatic_r0', 'low', float(pars[0]))
-                self.set('symptomatic_r0', 'high', float(pars[0]))
-            except Exception:
-                raise ValueError(
-                    f'The symptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[0]} provided'
-                )
+            self.set('symptomatic_r0', 'low', as_float(
+                pars[0], "The symptomatic_r0 should be a float number, if it is not a multiplier for groups"))
+            self.set('symptomatic_r0', 'high', as_float(pars[0]))
         elif len(pars) == 2:
-            try:
-                self.set('symptomatic_r0', 'low', float(pars[0]))
-            except Exception:
-                raise ValueError(
-                    f'The symptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[0]} provided'
-                )
-            try:
-                self.set('symptomatic_r0', 'high', float(pars[1]))
-            except Exception:
-                raise ValueError(
-                    f'The symptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[1]} provided'
-                )
+            self.set('symptomatic_r0', 'low', as_float(
+                pars[0], "The symptomatic_r0 should be a float number, if it is not a multiplier for groups"))
+            self.set('symptomatic_r0', 'high', as_float(
+                pars[1], "The symptomatic_r0 should be a float number, if it is not a multiplier for groups"))
+
         elif len(pars) > 2:
             raise ValueError(
                 f'The symptomatic_r0 should be one or two float number.')
         #
         for multiplier in [x for x in val if '=' in x]:
-            name, value = multiplier.split('=', 1)
-            try:
-                value = float(value)
-            except Exception:
-                raise ValueError(
-                    f'Multiplier should have format name=float_value: {multiplier} provided'
-                )
-            self.set('symptomatic_r0', f'multiplier_{name}', value)
+            self._set_multiplier(multiplier, 'symptomatic_r0')
 
     def set_asymptomatic_r0(self, val):
         if not val:
@@ -136,39 +119,20 @@ class Params:
         # if asymptomatic_r0 is specified, it should a number of a range...
         pars = [x for x in val if '=' not in x]
         if len(pars) == 1:
-            try:
-                self.set('asymptomatic_r0', 'low', float(pars[0]))
-                self.set('asymptomatic_r0', 'high', float(pars[0]))
-            except Exception:
-                raise ValueError(
-                    f'The asymptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[0]} provided'
-                )
+            self.set('asymptomatic_r0', 'low', as_float(
+                pars[0], "The asymptomatic_r0 should be a float number, if it is not a multiplier for groups"))
+            self.set('asymptomatic_r0', 'high', as_float(pars[0]))
         elif len(pars) == 2:
-            try:
-                self.set('asymptomatic_r0', 'low', float(pars[0]))
-            except Exception:
-                raise ValueError(
-                    f'The asymptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[0]} provided'
-                )
-            try:
-                self.set('asymptomatic_r0', 'high', float(pars[1]))
-            except Exception:
-                raise ValueError(
-                    f'The asymptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[1]} provided'
-                )
+            self.set('asymptomatic_r0', 'low', as_float(
+                pars[0], "The asymptomatic_r0 should be a float number, if it is not a multiplier for groups"))
+            self.set('asymptomatic_r0', 'high', as_float(
+                pars[1], "The asymptomatic_r0 should be a float number, if it is not a multiplier for groups"))
         elif len(pars) > 2:
             raise ValueError(
                 f'The asymptomatic_r0 should be one or two float number.')
         #
         for multiplier in [x for x in val if '=' in x]:
-            name, value = multiplier.split('=', 1)
-            try:
-                value = float(value)
-            except Exception:
-                raise ValueError(
-                    f'Multiplier should have format name=float_value: {multiplier} provided'
-                )
-            self.set('asymptomatic_r0', f'multiplier_{name}', value)
+            self._set_multiplier(multiplier, 'asymptomatic_r0')
 
     def set_incubation_period(self, val):
         if not val:
@@ -184,49 +148,32 @@ class Params:
                 raise ValueError(
                     f'Only normal or lognormal distribution for incubation period is supported. {pars[0]} provided'
                 )
-            try:
-                self.set('incubation_period',
-                         'mean' if pars[0] == 'lognormal' else 'loc',
-                         float(pars[1]))
-            except Exception:
-                raise ValueError(
-                    f'Second parameter of incubation_period should be a float number: {pars[1]} provided'
-                )
-            try:
-                self.set('incubation_period',
-                         'sigma' if pars[0] == 'lognormal' else 'scale',
-                         float(pars[2]))
-            except Exception:
-                raise ValueError(
-                    f'Third parameter of lognormal incubation_period should be a float number: {pars[2]} provided'
-                )
+            self.set('incubation_period',
+                     'mean' if pars[0] == 'lognormal' else 'loc',
+                     as_float(pars[1], "Second parameter of incubation_period should be a float number"))
+            self.set('incubation_period',
+                     'sigma' if pars[0] == 'lognormal' else 'scale',
+                     as_float(pars[2], "Third parameter of lognormal incubation_period should be a float number"))
         # multipliers
         for multiplier in [x for x in val if '=' in x]:
-            name, value = multiplier.split('=', 1)
-            try:
-                value = float(value)
-            except Exception:
-                raise ValueError(
-                    f'Multiplier should have format name=float_value: {multiplier} provided'
-                )
-            self.set('incubation_period', f'multiplier_{name}', value)
+            self._set_multiplier(multiplier, 'incubation_period')
 
     def set_susceptibility(self, val):
         if not val:
             return
-        for multiplier in val:
-            if '=' not in multiplier:
-                raise ValueError(
-                    f'Susceptibility has to be specified as name=weight: {multiplier} specified.'
-                )
-            name, value = multiplier.split('=', 1)
-            try:
-                value = float(value)
-            except Exception:
-                raise ValueError(
-                    f'Multiplier should have format name=float_value: {multiplier} provided'
-                )
-            self.set('susceptibility', f'multiplier_{name}', value)
+        base = [x for x in val if not isinstance(x, str) or '=' not in x]
+        if not base:
+            self.set('susceptibility', 'mean', 1.0)
+        elif len(base) > 1:
+            raise ValueError('Susceptibility should be a single number.')
+        else:
+            base_val = as_float(base[0], 'Susceptibility should be a float number')
+            if base_val > 1 or base_val < 0:
+                raise ValueError('suscepbility should be betwee 0 and 1: {base_val}')
+            self.set('susceptibility', 'mean', base_val)
+
+        for multiplier in [x for x in val if isinstance(x, str) and '=' in x]:
+            self._set_multiplier(multiplier, 'susceptibility')
 
     def set_prop_asym_carriers(self, val):
         if not val:
@@ -238,7 +185,7 @@ class Params:
                 f'Paramter prop-asym-carriers expect one or two float value or multipliers: {" ".join(val)} provided'
             )
         if len(pars) == 1:
-            self.set('prop_asym_carriers', 'loc', float(pars[0]))
+            self.set('prop_asym_carriers', 'loc', as_float(pars[0]))
             self.set('prop_asym_carriers', 'scale', 0)
         elif len(pars) == 2:
             if pars[0] > pars[1]:
@@ -252,14 +199,8 @@ class Params:
                 f'Parameter prop-asym-carriers accepts one or two numbers.')
         #
         for multiplier in [x for x in val if '=' in x]:
-            name, value = multiplier.split('=', 1)
-            try:
-                value = float(value)
-            except Exception:
-                raise ValueError(
-                    f'Multiplier should have format name=float_value: {multiplier} provided'
-                )
-            self.set('prop_asym_carriers', f'multiplier_{name}', value)
+            self._set_multiplier(multiplier, 'prop_asym_carriers')
+
 
     def set_params(self, args):
         # set some default values first
