@@ -2,7 +2,6 @@ import random
 
 from covid19_outbreak_simulator.event import Event, EventType
 from covid19_outbreak_simulator.plugin import BasePlugin
-from covid19_outbreak_simulator.population import Individual
 
 
 class remove(BasePlugin):
@@ -22,8 +21,8 @@ class remove(BasePlugin):
             'popsize',
             nargs='+',
             help='''Number of individuals to be removed, which can be from
-            the entire population, or from specific subpopulations
-            (e.g. nurse=10) if there are subpopulations.''')
+            the entire population, or from specific groups
+            (e.g. nurse=10) if there are groups.''')
         return parser
 
     def apply(self, time, population, args=None):
@@ -32,12 +31,12 @@ class remove(BasePlugin):
             name = ps.split('=', 1)[0] if '=' in ps else ''
             sz = int(ps.split('=', 1)[1]) if '=' in ps else int(ps)
 
-            if name not in population.subpop_sizes:
+            if name not in population.group_sizes:
                 raise ValueError(f'Subpopulation {name} does not exist')
 
             IDs = [x for x in population.ids if x.startswith(name) and x[len(name):].isnumeric()]
             random.shuffle(IDs)
             for ID in IDs[:sz]:
-                population.remove(ID, subpop=name)
+                population.remove(ID)
             self.logger.write(f'{self.logger.id}\t{time:.2f}\tREMOVE\t.\tsubpop={name},size={sz},IDs={",".join(IDs[:sz])}\n')
         return events
