@@ -1,17 +1,17 @@
-import random
+import copy
 import numpy as np
-from numpy.random import choice
-
+from numpy.random import choice, rand
+from fnmatch import fnmatch
+from .utils import as_float
 from .event import Event, EventType
 import re
 
 class Individual(object):
 
-    def __init__(self, id, group, susceptibility, model, logger):
+    def __init__(self, id, susceptibility, model, logger):
         self.id = id
         self.model = model
-        self.group = group
-        self.susceptibility = 1.0 if susceptibility is None else susceptibility
+        self.susceptibility = 1.0 if susceptibility is None else min(1, susceptibility)
         self.logger = logger
 
         # these will be set to event happen time
@@ -24,6 +24,10 @@ class Individual(object):
         self.r0 = None
         self.incubation_period = None
 
+    @property
+    def group(self):
+        return self.id.rsplit('_', 1)[0] if '_' in self.id else ''
+
     def __str__(self):
         return self.id
 
@@ -64,12 +68,7 @@ class Individual(object):
             elif kwargs['leadtime'] == 'asymptomatic':
                 lead_time = np.random.uniform(0, self.incubation_period)
             else:
-                try:
-                    lead_time = float(kwargs['leadtime'])
-                except:
-                    raise ValueError(
-                        f'--leadtime can only be any, asymptomatic, or a fixed number: {kwargs["leadtime"]} provided'
-                    )
+                lead_time = as_float(kwargs['leadtime'], "--leadtime can only be any, asymptomatic, or a fixed number")
         else:
             lead_time = 0
 
@@ -102,12 +101,7 @@ class Individual(object):
             if len(handle_symptomatic) == 1:
                 proportion = 1
             else:
-                try:
-                    proportion = float(handle_symptomatic[1])
-                except Exception:
-                    raise ValueError(
-                        f'Proportion in "--handle-symptomatic remove/keep prop" should be a float number: {handle_symptomatic[1]} provided'
-                    )
+                proportion = as_float(handle_symptomatic[1], "Proportion in --handle-symptomatic remove/keep prop should be a float number")
             if proportion > 1 or proportion < 0:
                 raise ValueError(
                     f'Proportion in "--handle-symptomatic remove/keep prop" should be a float number between 0 and 1: {proportion} provided'
@@ -136,22 +130,12 @@ class Individual(object):
             if handle_symptomatic[0] == 'quarantine':
                 quarantine_duration = 14
             else:
-                try:
-                    quarantine_duration = float(handle_symptomatic[0].split(
-                        '_', 1)[1])
-                except Exception:
-                    raise ValueError(
-                        f'quanrantine duration should be specified as "quarantine_DURATION": {handle_symptomatic[0]} provided'
-                    )
+                quarantine_duration = as_float(handle_symptomatic[0].split(
+                        '_', 1)[1], "quanrantine duration should be specified as quarantine_DURATION")
             if len(handle_symptomatic) == 1:
                 proportion = 1
             else:
-                try:
-                    proportion = float(handle_symptomatic[1])
-                except Exception:
-                    raise ValueError(
-                        f'Proportion in "--handle-symptomatic quarantine_DURATION prop" should be a float number: {handle_symptomatic[1]} provided'
-                    )
+                proportion = as_float(handle_symptomatic[1], "Proportion in --handle-symptomatic quarantine_DURATION prop should be a float number")
             if proportion > 1 or proportion < 0:
                 raise ValueError(
                     f'Proportion in "--handle-symptomatic quarantine_DUURATION prop" should be a float number between 0 and 1: {proportion} provided'
@@ -261,12 +245,7 @@ class Individual(object):
                 # could be anywhere in his incubation period
                 lead_time = np.random.uniform(0, 10)
             else:
-                try:
-                    lead_time = float(kwargs['leadtime'])
-                except:
-                    raise ValueError(
-                        f'--leadtime can only be any, asymptomatic, or a fixed number: {kwargs["leadtime"]} provided'
-                    )
+                lead_time = as_float(kwargs['leadtime'], "--leadtime can only be any, asymptomatic, or a fixed number")
         else:
             lead_time = 0
 
@@ -340,6 +319,13 @@ class Individual(object):
             )
             return []
 
+        if self.susceptibility < 1 and rand() > self.susceptibility:
+            by_id = '.' if kwargs['by'] is None else kwargs['by']
+            self.logger.write(
+                f'{self.logger.id}\t{time:.2f}\t{EventType.INFECTION_FAILED.name}\t{self.id}\tby={by_id},reson=susceptibility\n'
+            )
+            return []
+
         if self.model.draw_is_asymptomatic():
             return self.asymptomatic_infect(time, **kwargs)
         else:
@@ -348,34 +334,101 @@ class Individual(object):
 
 class Population(object):
 
-    def __init__(self, popsize, uneven_susceptibility=False):
+    def __init__(self, popsize, model, vicinity, logger):
         self.individuals = {}
-        self.subpop_sizes = {(ps.split('=', 1)[0] if '=' in ps else ''):
+        self.group_sizes = {(ps.split('=', 1)[0] if '=' in ps else ''):
                         0 for ps in popsize}
-        self.max_ids = {x:y for x,y in self.subpop_sizes.items() }
-        self.uneven_susceptibility = uneven_susceptibility
+        self.max_ids = {x:y for x,y in self.group_sizes.items() }
         self.subpop_from_id = re.compile('^(.*?)[\d]+$')
+        self.vicinity = self.parse_vicinity(vicinity)
+
+        idx = 0
+
+        for ps in popsize:
+            if '=' in ps:
+                # this is named population size
+                name, sz = ps.split('=', 1)
+            else:
+                name = ''
+                sz = ps
+            try:
+                sz = int(sz)
+            except Exception:
+                raise ValueError(
+                    f'Named population size should be name=int: {ps} provided')
+
+            self.add([
+                Individual(
+                    f'{name}_{idx}' if name else str(idx),
+                        susceptibility=getattr(model.params,
+                                           f'susceptibility_mean',
+                                           1) * getattr(model.params,
+                                           f'susceptibility_multiplier_{name}',
+                                           1),
+                    model=model,
+                    logger=logger) for idx in range(idx, idx + sz)
+            ], subpop=name)
+
+    def parse_vicinity(self, params):
+        if not params:
+            return {}
+
+        res = {}
+        for param in params:
+            matched = re.match('^(.*)-(.*)=(\d+)$', param)
+            if matched:
+                infector_sp = matched.group(1)
+                infectee_sp = matched.group(2)
+                neighbor_size = int(matched.group(3))
+            else:
+                matched = re.match('^(.*)=(\d+)$', param)
+                if matched:
+                    infector_sp = ''
+                    infectee_sp = matched.group(1)
+                    neighbor_size = int(matched.group(2))
+                if not matched:
+                    raise ValueError(f'Vicinity should be specified as "INFECTOR_SO-INFECTEE_SP=SIZE": {param} specified')
+
+            if infector_sp == '':
+                infector_sps = ['']
+            elif infector_sp.startswith('!'):
+                infector_sps = [x for x in self.group_sizes.keys() if not fnmatch(x, infector_sp[1:])]
+            else:
+                infector_sps = [x for x in self.group_sizes.keys() if fnmatch(x, infector_sp)]
+
+            if infectee_sp.startswith('!'):
+                infectee_sps = [x for x in self.group_sizes.keys() if not fnmatch(x, infectee_sp[1:])]
+            else:
+                infectee_sps = [x for x in self.group_sizes.keys() if fnmatch(x, infectee_sp)]
+
+            if infector_sp != '' and not infector_sps:
+                raise ValueError(f'Unrecognized group {infector_sp}')
+            if not infectee_sps:
+                raise ValueError(f'Unrecognized group {infectee_sp}')
+
+            for infector_sp in infector_sps:
+                for infectee_sp in infectee_sps:
+                    if infector_sp in res:
+                        res[infector_sp][infectee_sp] = neighbor_size
+                    else:
+                        res[infector_sp] = {infectee_sp: neighbor_size}
+        return res
 
     def add(self, items, subpop):
         sz = len(self.individuals)
         self.individuals.update({x.id: x for x in items})
         if sz + len(items) != len(self.individuals):
             raise ValueError(f'One or more IDs are already in the population.')
-        self.subpop_sizes[subpop] += len(items)
+        self.group_sizes[subpop] += len(items)
         self.max_ids[subpop] += len(items)
 
     @property
     def ids(self):
         return self.individuals.keys()
 
-    def remove(self, item, subpop=None):
+    def remove(self, item):
+        self.group_sizes[self.individuals[item].group] -= 1
         self.individuals.pop(item)
-        if subpop is None:
-            subpop = self.subpop_from_id.match(item).group(1)
-        self.subpop_sizes[subpop] -= 1
-
-    def in_subpop(self, item, subpop):
-        return True if not subpop else self.subpop_from_id.match(item).group(1) == subpop
 
     def __len__(self):
         return len(self.individuals)
@@ -392,25 +445,45 @@ class Population(object):
     def values(self):
         return self.individuals.values()
 
-    def select(self, exclude):
-        if self.uneven_susceptibility:
-            # select one non-quarantined indivudal to infect
-            ids = [(id, ind.susceptibility)
-                   for id, ind in self.individuals.items()
-                   if id != exclude and not ind.quarantined]
+    def select(self, infector=None):
+        # select one non-quarantined indivudal to infect
+        #
+        if infector is not None and infector not in self.individuals:
+            raise RuntimeError(f'Can not select infectee if since infector {infector} no longer exists.')
 
-            if not ids:
-                return None
-
-            weights = np.array([x[1] for x in ids])
-            weights = weights / sum(weights)
-            return ids[choice(len(ids), 1, p=weights)[0]][0]
-        else:
-            # select one non-quarantined indivudal to infect
+        # if not cicinity is defines, or
+        # if infection is from community and '' not in vicinity, or
+        # with infector but self.vicinity does not have this case.
+        if not self.vicinity or (infector is None and '' not in self.vicinity) or \
+            (infector is not None and self.individuals[infector].group not in self.vicinity):
             ids = [
                 id for id, ind in self.individuals.items()
-                if id != exclude and not ind.quarantined
+                if infector != ind.id and not ind.quarantined
             ]
-            if not ids:
-                return None
-            return random.choice(ids)
+        else:
+            # quota from each group.
+            groups = list(self.group_sizes.keys())
+
+            if infector in self.individuals:
+                freq = copy.deepcopy(self.vicinity[self.individuals[infector].group])
+            else:
+                freq = copy.deepcopy(self.vicinity[''])
+
+            for grp in self.group_sizes.keys():
+                if grp not in freq:
+                    freq[grp] = self.group_sizes[grp]
+                    # now, we know the number of qualified invidiauls from each
+                    # group, but we still have excluded and quarantined....
+            total = sum(freq.values())
+            freq = {x:y/total for x,y in freq.items()}
+            # first determine which group ...
+            grp = choice(groups, 1, p=[freq[x] for x in groups])
+
+            # then select a random individual from the group.
+            ids = [id for id, ind in self.individuals.items()
+                if ind.group == grp and infector != ind.id and not ind.quarantined
+            ]
+
+        if not ids:
+            return None
+        return choice(ids)
