@@ -1,11 +1,11 @@
 import pytest
 from itertools import product
-from covid19_outbreak_simulator.simulator import Individual
+from covid19_outbreak_simulator.population import Individual
 from covid19_outbreak_simulator.event import EventType
 
 
 def test_individual(default_model, logger):
-    ind = Individual(0, 'group 1', 1.2, default_model, logger)
+    ind = Individual('group1_0', 1.2, default_model, logger)
 
     assert ind.infected is False
     assert ind.quarantined is False
@@ -36,12 +36,33 @@ def test_quarantine_error(individual):
 
 def test_infect(individual):
     individual.model.draw_prop_asym_carriers()
-    individual.infect(5.0, by=None)
+    #
+    res = []
+    for x in range(1000):
+        individual.infected = None
+        res += individual.infect(5.0, by=None)
+    regular = len(res)
+    assert res
+
+    individual.susceptibility = 0
+    res = []
+    for x in range(100):
+        individual.infected = None
+        res += individual.infect(5.0, by=None)
+    assert not res
+
+    # 80% of infection events would fail
+    individual.susceptibility = 0.2
+    res = []
+    for x in range(1000):
+        individual.infected = None
+        res += individual.infect(5.0, by=None)
+    assert len(res) > regular * 0.10 and len(res) < regular * 0.30
 
 
 def test_infect_infected(individual_factory):
-    ind = individual_factory(id=1)
-    ind1 = individual_factory(id=3)
+    ind = individual_factory(id='1')
+    ind1 = individual_factory(id='3')
     ind.model.draw_prop_asym_carriers()
     ind.infected = 5.5
 
